--[[
Interface: 1.4.1.0 b5334

Author: GtX | Andy
Date: 02.02.2019
Version: 1.0.0.0

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

History:
V 1.0.0.0 @ 02.02.2019 - Release Version

About:
Simple loader script for placeables. This saves having 2 or 3 loader scripts in maps or mods for custom scripts.
Unfortunately not supported in modDesc as it should be, but now it is. ;-)

Example:

<modDesc>
    <placeables>
        <placeable typeName="myPlaceable" className="MyPlaceableClass" filename="scripts/MyPlaceableScript.lua"/>
    </placeables>
</modDesc>
]]


PlaceableLoader = {}

local modName = g_currentModName
local modDirectory = g_currentModDirectory

function PlaceableLoader.init()
    PlaceableLoader.loaded = {}

    local xmlFile = loadXMLFile("modDesc_placeableLoader", modDirectory .. "modDesc.xml")
    if xmlFile ~= 0 then

        local i = 0
        while true do
            local key = string.format("modDesc.placeables.placeable(%d)", i)
            if not hasXMLProperty(xmlFile, key) then
                break
            end

            local isValid = true

            local typeName = getXMLString(xmlFile, key .. "#typeName")
            if typeName == nil then
                isValid = PlaceableLoader.errorPrint("typeName", modName, key)
            end

            local className = getXMLString(xmlFile, key .. "#className")
            if className == nil then
                isValid = PlaceableLoader.errorPrint("className", modName, key)
            end

            local filename = getXMLString(xmlFile, key .. "#filename")
            if filename == nil then
                isValid = PlaceableLoader.errorPrint("filename", modName, key)
            end

            if isValid and not PlaceableLoader.loaded[typeName] then
                local placeablePath = modDirectory .. filename

                if fileExists(placeablePath) then
                    g_placeableTypeManager:addPlaceableType(typeName, className, placeablePath)

                    PlaceableLoader.loaded[typeName] = true
                else
                    print(string.format("  Error: [%s] File %s could not be found!", modName, placeablePath))
                end
            end

            i = i + 1
        end

        delete(xmlFile)
        xmlFile = nil
    end
end

function PlaceableLoader.errorPrint(name, modName, key)
    print(string.format("  Error: [%s] Missing '%s' at %s. Placeable will not be added!", modName, name, key))
    return false
end

PlaceableLoader.init()


-- ExtraFillUnit = {}

-- function ExtraFillUnit:onDraw(isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	-- local spec = self.spec_fillUnit
	-- if spec.unloading ~= nil then
	    -- for _, unloading in ipairs(spec.unloading) do
	        -- local node = unloading.node
	        -- DebugUtil.drawDebugNode(node, "n", false)
	        -- local ox, oy, oz = unpack(unloading.offset)
	        -- local x, y, z = localToWorld(node, ox - unloading.width*0.5, oy, oz)
	        -- local place = {}
	        -- place.startX, place.startY, place.startZ = x, y, z
	        -- place.rotX, place.rotY, place.rotZ = getWorldRotation(node)
	        -- place.dirX, place.dirY, place.dirZ = localDirectionToWorld(node, 1, 0, 0)
	        -- place.dirPerpX, place.dirPerpY, place.dirPerpZ = localDirectionToWorld(node, 0, 0, 1)
	        -- place.yOffset = 1
	        -- place.maxWidth = math.huge
	        -- place.maxLength = math.huge
	        -- place.width = unloading.width
	        -- drawDebugLine(place.startX, place.startY, place.startZ, 1, 0, 0, place.startX+place.dirX*place.width, place.startY+place.dirY*place.width, place.startZ+place.dirZ*place.width, 1, 0, 0)
	        -- DebugUtil.drawDebugGizmoAtWorldPos(place.startX, place.startY, place.startZ, place.dirX, place.dirY, place.dirZ, 0, 1, 0, "l")
	        -- DebugUtil.drawDebugGizmoAtWorldPos(place.startX+place.dirX*place.width, place.startY+place.dirY*place.width, place.startZ+place.dirZ*place.width, place.dirX, place.dirY, place.dirZ, 0, 1, 0, "r")
	    -- end
	-- end
	
	
	-- local spec = self.spec_dischargeable
	-- if spec ~= nil then
		-- local values = {}
		
		-- local currentDischargeNode = spec.currentDischargeNode
		-- local state = "OFF"
		-- if spec.currentDischargeState == Dischargeable.DISCHARGE_STATE_OBJECT then
			-- state = "OBJECT"
		-- elseif spec.currentDischargeState == Dischargeable.DISCHARGE_STATE_GROUND then
			-- state = "GROUND"
		-- end
		-- table.insert(values, {name="state", value=state})
		-- table.insert(values, {name="getCanDischargeToObject", value=tostring(self:getCanDischargeToObject(currentDischargeNode))})
		-- table.insert(values, {name="getCanDischargeToGround", value=tostring(self:getCanDischargeToGround(currentDischargeNode))})
		-- table.insert(values, {name="dischargedLiters", value=tostring(spec.dischargedLiters)})
		-- table.insert(values, {name="currentNode", value=tostring(currentDischargeNode)})
		-- for _, dischargeNode in ipairs(spec.dischargeNodes) do
			-- table.insert(values, {name="--->", value=tostring(dischargeNode)})
			-- local object = nil
			-- if dischargeNode.dischargeObject ~= nil then
				-- object = tostring(dischargeNode.dischargeObject.configFileName)
			-- end
			-- table.insert(values, {name="object", value=tostring(object)})
			-- table.insert(values, {name="distance", value=dischargeNode.dischargeDistance})
			-- table.insert(values, {name="effect", value=tostring(dischargeNode.isEffectActive)})
			-- table.insert(values, {name="fillLevel", value=tostring(self:getFillUnitFillLevel(dischargeNode.fillUnitIndex))})
			-- table.insert(values, {name="litersToDrop", value=tostring(dischargeNode.litersToDrop)})
			-- table.insert(values, {name="emptyFactor", value=tostring(self:getDischargeNodeEmptyFactor(dischargeNode))})
			-- table.insert(values, {name="emptySpeed", value=tostring(self:getDischargeNodeEmptyFactor(dischargeNode))})
			-- table.insert(values, {name="readyForDischarge", value=tostring(dischargeNode.lastEffect == nil or dischargeNode.lastEffect:getIsFullyVisible())})
			-- table.insert(values, {name="objectsInTrigger", value=tostring(dischargeNode.trigger.numObjects)})
			-- table.insert(values, {name="objectsInActivationTrigger", value=tostring(dischargeNode.activationTrigger.numObjects)})
		-- end
		
		-- DebugUtil.renderTable(0.25, 0.9, 0.02, values, 0.05)
	-- end	
-- end
-- FillUnit.onDraw = Utils.appendedFunction(FillUnit.onDraw, ExtraFillUnit.onDraw)


-- function Trailer:getDischargeNodeEmptyFactor(superFunc, dischargeNode)
    -- local spec = self.spec_trailer
    -- local tipSide = spec.dischargeNodeIndexToTipSide[dischargeNode.index]
    -- if tipSide ~= nil then
        -- if tipSide.animation.name ~= nil and tipSide.animation.startTipTime ~= 0 then
            -- log(self:getAnimationTime(tipSide.animation.name), tipSide.animation.startTipTime )
            -- if self:getAnimationTime(tipSide.animation.name) < tipSide.animation.startTipTime then
                -- return 0
            -- end
        -- end
        -- if tipSide.doorAnimation.name ~= nil and tipSide.doorAnimation.startTipTime ~= 0 then
            -- if self:getAnimationTime(tipSide.doorAnimation.name) < tipSide.doorAnimation.startTipTime then
                -- return 0
            -- end
        -- end
        -- return tipSide.currentEmptyFactor
    -- end
    -- return superFunc(self, dischargeNode)
-- end




















